/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "asset.hpp"

namespace nostalgia::foundation {

ox::Result<ox::UUID> readUuidHeader(const ox::Buffer &buff) noexcept {
	return readUuidHeader(buff.data(), buff.size());
}

ox::Result<ox::UUID> readUuidHeader(const char *buff, std::size_t buffLen) noexcept {
	if (buffLen < 40) {
		return OxError(1, "Insufficient data contain complete Nostalgia header");
	}
	if (ox_memcmp(buff, "N1;", 3) != 0) {
		return OxError(2, "No Nostalgia header data");
	}
	return ox::UUID::fromString(ox::StringView(buff + 3, 36));
}

ox::Result<ox::ModelObject> readAsset(ox::TypeStore *ts, const ox::Buffer &buff) noexcept {
	std::size_t offset = 0;
	if (!readUuidHeader(buff).error) {
		offset = 40; // the size of N1 headers
	}
	return ox::readClaw(ts, buff.data() + offset, buff.size() - offset);
}

ox::Result<AssetHdr> readAssetHeader(const char *buff, std::size_t buffLen) noexcept {
	AssetHdr out;
	const auto err = readUuidHeader(buff, buffLen).moveTo(&out.uuid);
	const auto offset = err ? 0 : 40;
	buff = buff + offset;
	buffLen = buffLen - offset;
	oxReturnError(ox::readClawHeader(buff, buffLen).moveTo(&out.clawHdr));
	return out;
}

ox::Result<AssetHdr> readAssetHeader(const ox::Buffer &buff) noexcept {
	return readAssetHeader(buff.data(), buff.size());
}

}
